;#AutoIt3Wrapper_UseX64=y

#include ".\Includes\CLR.au3"

Example()

Func Example()
  ; Compile the helper class. This could be pre-compiled.
  Local $oHelper, $oHelperAsm = _CLR_CompileCSharp( FileRead( "EventHelper.cs" ) )
  $oHelperAsm.CreateInstance( "EventHelper", $oHelper )
  ConsoleWrite( "EventHelper.cs -> IsObj( $oHelper ) = " & IsObj( $oHelper ) & @CRLF )

  ; Create our test object, which simply exposes a single event.
  Local $oTest, $oTestAsm = _CLR_CompileCSharp( FileRead( "TestObject.cs" ) )
  $oTestAsm.CreateInstance( "ObjectWithEvent", $oTest )
  ConsoleWrite( "TestObject.cs -> IsObj( $oTest ) = " & IsObj( $oTest ) & @CRLF )

  Local $hEventHandler = DllCallbackRegister( "EventHandler", "int", "ptr" )
  Local $pEventHandler = DllCallbackGetPtr( $hEventHandler )

  ; Add an event handler for the "OnEvent" event.  Use "" to pass the
  ; address as a string, since IDispatch doesn't support 64-bit integers.
  $oHelper.AddHandler( $oTest, "OnEvent", $pEventHandler )

  ; Make an event handler (event must be of the EventHandler type).
  Local $oHandler = $oHelper.MakeHandler( $pEventHandler )
  $oTest.add_OnEvent( $oHandler )

  ; Test the event handlers.
  $oTest.RaiseEvent()
EndFunc

; Our event handler is called with a SAFEARRAY of parameters.  This
; makes it much easier to get the type and value of each parameter.
Func EventHandler( $pprm )
  ConsoleWrite( "$pprm = " & $pprm & @CRLF )

  Local $iDim = SafeArrayGetDim( $pprm )
  ConsoleWrite( "$iDim = " & $iDim & @CRLF )

  Local $iLBound, $iUBound
  SafeArrayGetLBound( $pprm, 1, $iLBound )
  SafeArrayGetUBound( $pprm, 1, $iUBound )
  ConsoleWrite( "$iLBound = " & $iLBound & @CRLF )
  ConsoleWrite( "$iUBound = " & $iUBound & @CRLF )

  Local $tprm = DllStructCreate( $tagSAFEARRAY, $pprm )
  Local $fFeatures = DllStructGetData( $tprm, "fFeatures" )
  ConsoleWrite( "$fFeatures = 0x" & Hex( $fFeatures ) & @CRLF )
  Local $cbElements = DllStructGetData( $tprm, "cbElements" )
  ConsoleWrite( "$cbElements = " & $cbElements & @CRLF )

  Local $vt
  SafeArrayGetVartype( $pprm, $vt )
  ConsoleWrite( "$vt = " & $vt & @CRLF )

  Local $prmData, $tvt, $data, $obj
  SafeArrayAccessData( $pprm, $prmData )

  $tvt = DllStructCreate( "word", $prmData )
  $vt = DllStructGetData( $tvt, 1 )
  ConsoleWrite( "$vt = " & $vt & @CRLF ) ; EventArgs Class
  $data = DllStructGetData( DllStructCreate( "ptr", $prmData + 8 ), 1 )
  $obj = ConvertPtrToIDispatch($data)
  ConsoleWrite( $obj.ToString() & @CRLF )

  $tvt = DllStructCreate( "word", $prmData + ( @AutoItX64 ? 24 : 16 ) )
  $vt = DllStructGetData( $tvt, 1 )
  ConsoleWrite( "$vt = " & $vt & @CRLF ) ; EventArgs Class
  $data = DllStructGetData( DllStructCreate( "ptr", $prmData + ( @AutoItX64 ? 24 : 16 ) + 8 ), 1 )
  $obj = ConvertPtrToIDispatch($data)
  ConsoleWrite( $obj.ToString() & @CRLF )

  SafeArrayUnaccessData( $pprm )
EndFunc

; In the end we still want the autoit object. This function converts a raw pointer to an autoit object
Func ConvertPtrToIDispatch($IDispatch_Ptr)
  ; This would have been 10000x easier if autoit had supported the idispatch* type in dllstructs...
  ; Fortunetely memcpy can copy the pointer into a idispatch*, lucky us.
  $ptr_struct=DllStructCreate("ptr")
  DllStructSetData($ptr_struct,1,$IDispatch_Ptr)
  $aCall = DllCall("ntdll.dll","ptr:cdecl","memcpy","idispatch*","","ptr",DllStructGetPtr($ptr_struct),"long",4)
  return $aCall[1]
EndFunc