;#AutoIt3Wrapper_UseX64=y

#include ".\Includes\CLR.au3"

Example() ; Form Using System.Windows.Forms.Form

Func Example()
  ; Compile the helper class. This could be pre-compiled.
  Local $oHelper, $oHelperAsm = _CLR_CompileCSharp( FileRead( "EventHelper.cs" ) )
  $oHelperAsm.CreateInstance( "EventHelper", $oHelper )
  ConsoleWrite( "IsObj( $oHelper ) = " & IsObj( $oHelper ) & @CRLF )

  ConsoleWrite( "_CLR_LoadLibrary System.Windows.Forms" & @CRLF )
  Local $oAssembly = _CLR_LoadLibrary( "System.Windows.Forms" )
  ConsoleWrite( "IsObj( $oAssembly ) = " & IsObj( $oAssembly ) & @CRLF )

  ConsoleWrite( @CRLF & "_CLR_CreateObject: System.Windows.Forms.Form" & @CRLF )
  Local $oForm = _CLR_CreateObject( $oAssembly, "System.Windows.Forms.Form" )
  ConsoleWrite( "IsObj( $oForm ) = " & IsObj( $oForm ) & @CRLF )

  $oForm.Text = "Form From Net - AutoIt Rocks"
  $oForm.Width = 800
  $oForm.Height = 400

  ConsoleWrite( @CRLF & "_CLR_CreateObject System.Windows.Forms.Button" & @CRLF )
  Local $oButton1 = _CLR_CreateObject($oAssembly, "System.Windows.Forms.Button")
  $oButton1.Text = "button1"
  $oButton1.Width = 60
  $oButton1.Height = 30
  $oButton1.Left = 100
  $oButton1.Top = 100

  ConsoleWrite( @CRLF & "_CLR_CreateObject System.Windows.Forms.Button" & @CRLF )
  Local $oButton2 = _CLR_CreateObject($oAssembly, "System.Windows.Forms.Button")
  $oButton2.Text = "button2"
  $oButton2.Width = 60
  $oButton2.Height = 30
  $oButton2.Left = $oButton1.Right + 50
  $oButton2.Top = $oButton1.Top

  Local $hEventHandler = DllCallbackRegister( "EventHandler", "int", "ptr" )
  Local $pEventHandler = DllCallbackGetPtr( $hEventHandler )

  ; Add an event handler for the "OnEvent" event.  Use "" to pass the
  ; address as a string, since IDispatch doesn't support 64-bit integers.
  $oHelper.AddHandler( $oButton1, "Click", $pEventHandler )
  $oHelper.AddHandler( $oButton2, "Click", $pEventHandler )

  ;$oForm.Controls.Add( $oButton1 ) ; ERR
  $oButton1.Parent = $oForm ; OK
  $oButton2.Parent = $oForm ; OK

  $oForm.ShowDialog()

  $oForm.Dispose()
EndFunc

; Our event handler is called with a SAFEARRAY of parameters.  This
; makes it much easier to get the type and value of each parameter.
Func EventHandler( $pprm )
  ConsoleWrite( "$pprm = " & $pprm & @CRLF )

  Local $iDim = SafeArrayGetDim( $pprm )
  ConsoleWrite( "$iDim = " & $iDim & @CRLF )

  Local $iLBound, $iUBound
  SafeArrayGetLBound( $pprm, 1, $iLBound )
  SafeArrayGetUBound( $pprm, 1, $iUBound )
  ConsoleWrite( "$iLBound = " & $iLBound & @CRLF )
  ConsoleWrite( "$iUBound = " & $iUBound & @CRLF )

  Local $tprm = DllStructCreate( $tagSAFEARRAY, $pprm )
  Local $fFeatures = DllStructGetData( $tprm, "fFeatures" )
  ConsoleWrite( "$fFeatures = 0x" & Hex( $fFeatures ) & @CRLF )
  Local $cbElements = DllStructGetData( $tprm, "cbElements" )
  ConsoleWrite( "$cbElements = " & $cbElements & @CRLF )

  Local $vt
  SafeArrayGetVartype( $pprm, $vt )
  ConsoleWrite( "$vt = " & $vt & @CRLF )

  Local $prmData, $tvt, $data, $obj
  SafeArrayAccessData( $pprm, $prmData )

  $tvt = DllStructCreate( "word", $prmData )
  $vt = DllStructGetData( $tvt, 1 )
  ConsoleWrite( "$vt = " & $vt & @CRLF ) ; EventArgs Class
  $data = DllStructGetData( DllStructCreate( "ptr", $prmData + 8 ), 1 )

    ;~   $obj = ConvertPtrToIDispatch($data)
  $obj = ObjCreateInterface($data, $sIID_IDispatch)

  Local $sCtrlInfo = $obj.ToString()
  ConsoleWrite( $sCtrlInfo & @CRLF )

  $tvt = DllStructCreate( "word", $prmData + ( @AutoItX64 ? 24 : 16 ) )
  $vt = DllStructGetData( $tvt, 1 )
  ConsoleWrite( "$vt = " & $vt & @CRLF ) ; EventArgs Class
  $data = DllStructGetData( DllStructCreate( "ptr", $prmData + ( @AutoItX64 ? 24 : 16 ) + 8 ), 1 )
;~   $obj = ConvertPtrToIDispatch($data)
  $obj = ObjCreateInterface($data, $sIID_IDispatch)
  ConsoleWrite( $obj.ToString() & @CRLF )

  SafeArrayUnaccessData( $pprm )

  Local $aCtrlInfo = StringSplit( $sCtrlInfo, ", ", 2 ) ; 2 = $STR_NOCOUNT
  For $i = 0 To UBound( $aCtrlInfo ) - 1
    ConsoleWrite( "$i = " & $i & ", $aCtrlInfo[$i] = " & $aCtrlInfo[$i] & @CRLF )
  Next
  Switch $aCtrlInfo[0]
    Case "System.Windows.Forms.Button"
      Switch $aCtrlInfo[3]
        Case "button1"
          MsgBox( 0, "", "button1" )
        Case "button2"
          MsgBox( 0, "", "button2" )
      EndSwitch
  EndSwitch
EndFunc

#cs
; In the end we still want the autoit object. This function converts a raw pointer to an autoit object
Func ConvertPtrToIDispatch($IDispatch_Ptr)
  ; This would have been 10000x easier if autoit had supported the idispatch* type in dllstructs...
  ; Fortunetely memcpy can copy the pointer into a idispatch*, lucky us.
  $ptr_struct=DllStructCreate("ptr")
  DllStructSetData($ptr_struct,1,$IDispatch_Ptr)
  $aCall = DllCall("ntdll.dll","ptr:cdecl","memcpy","idispatch*","","ptr",DllStructGetPtr($ptr_struct),"long",4)
  return $aCall[1]
EndFunc
#ce